r"""
This module is an interface for the NRAO faultchecker.

This file is meant to be run as a script from the command line.
"""
from __future__ import print_function

import argparse as ap
import logging
import os

import sys

from . import version
from .faultreporter import ConsoleReporter, BulkReporter
from .faultchecker import ServicesChecker, QueuesChecker, LastProcessedChecker, \
    DirChecker

logger = logging.getLogger(__name__)

# Description of this widget.
_DESCRIPTION = """Command line tool for checking failed workflow services on the AAT/PPI, version {}."""

# Printed at the end of the help for the tool, describes return values.
_EPILOG = """Return values:
1: can't deduce CAPO_PROFILE value
2: missing CAPO property
3: Application Exception
"""

# Error messages.
_MISSING_PROFILE = """ERROR: faultchecker can't deduce the 'profile', 
give it the -P argument or set the CAPO_PROFILE environment variable.\n\n"""
_MISSING_PROPERTY = "ERROR: missing required CAPO property {}\n\n"


def get_parser():
    r""" Build and return an argument parser for this tool.

    :return: Return an argument parser specific to faultchecker.
    """
    p = ap.ArgumentParser(description=_DESCRIPTION.format(version),
                          formatter_class=ap.RawTextHelpFormatter,
                          epilog=_EPILOG)
    p.add_argument('-P', '--profile', action='store',
                   help='profile name to use, e.g. test, production')
    return p


def die_message(message, parser, return_value):
    r""" Given an error message, argument parser and return_value, print the
        message, dump the parser's help and return with the return value.

    :param message: The error message to present to the user on application failure.
    :param parser: The parser from which to pull the help/usage from.
    :param return_value: The return value we're about to send to the OS.
    :return: None... we're dying.
    """
    """"""
    sys.stderr.write(message)
    parser.print_help()
    sys.exit(return_value)


def get_settings():
    r""" Get the settings necessary to run faultchecker.

    :return: The faultchecker configuration settings
    """
    parser = get_parser()
    args = parser.parse_args()
    return args


def check_services(agg, profile):
    try:
        sc = ServicesChecker(profile)
    except Exception:
        raise Exception('Unable to construct ServicesChecker({0})'.format(profile))

    if sc.check_servlet("amygdala"):
        agg.ok('{} servlet is running'.format("amygdala"))
    else:
        agg.error('{} servlet is NOT RUNNING!'.format("amygdala"))

    if sc.check_servlet("workflow"):
        agg.ok('{} servlet is running'.format("workflow"))
    else:
        agg.error('{} servlet is NOT RUNNING!'.format("workflow"))

    if sc.check_amqp_online():
        agg.ok('AMQP is online')
    else:
        agg.error('AMQP is NOT ONLINE!')


def check_queues(agg, profile):
    try:
        qc = QueuesChecker(profile)
    except Exception:
        raise Exception('Unable to construct QueuesChecker({0})'.format(profile))

    q_count = qc.get_queue_msg_count(qc.workflow.amqpPersistentQueueName)
    if q_count == 0:
        agg.ok('{} is empty'.format(qc.workflow.amqpPersistentQueueName))
    else:
        agg.error('{} is NOT EMPTY ({} messages in queue)'.format(qc.workflow.amqpPersistentQueueName, q_count))


def check_last_xxx(agg, profile):
    try:
        last_x = LastProcessedChecker(profile)
    except Exception:
        raise Exception('Unable to construct LastProcessedChecker({0})'.format(profile))

    last_bdf_ingest = last_x.get_last_ingested('bdf')
    if last_bdf_ingest.diff().in_hours() > 3:
        agg.error("last {} ingested {}".format('bdf'.upper(), last_bdf_ingest.diff_for_humans()))
    else:
        agg.ok("last {} ingested {}".format('bdf'.upper(), last_bdf_ingest.diff_for_humans()))

    last_sdm_ingest = last_x.get_last_ingested('sdm')
    if last_sdm_ingest.diff().in_hours() > 3:
        agg.error("last {} ingested {}".format('sdm'.upper(), last_sdm_ingest.diff_for_humans()))
    else:
        agg.ok("last {} ingested {}".format('sdm'.upper(), last_sdm_ingest.diff_for_humans()))

    last_eb = last_x.get_last_eb()
    if last_eb.diff().in_days() > 3:
        agg.error("last execution block ended {}".format(last_eb.diff_for_humans()))
    else:
        agg.ok("last execution block ended {}".format(last_eb.diff_for_humans()))

    count, last_alma_reingestion = last_x.get_last_alma_reingestion()
    if count > 0 and last_alma_reingestion.diff().in_minutes() > 5:
        agg.error("last ALMA SDM reingested {}".format(last_alma_reingestion.diff_for_humans()))
    else:
        agg.ok("last ALMA SDM reingested {}".format(last_alma_reingestion.diff_for_humans()))


def check_directories(agg, profile):
    try:
        dc = DirChecker(profile)
    except Exception:
        raise Exception('Unable to construct DirChecker({0})'.format(profile))

    if not os.path.exists(dc.sdmpath):
        agg.warn('Skipping SDM check: {} does not exist'.format(dc.sdmpath))
    else:
        missing_sdms = dc.get_missing_sdms()
        if len(missing_sdms) > 0:
            agg.error("Missing SDMs for {0}: ".format(dc.sdmpath) +
                      ', '.join(list(missing_sdms)))
        else:
            agg.ok("No missing recent SDMs")

    if not os.path.exists(dc.mchammer):
        agg.warn('Skipping SDM check: {} does not exist'.format(dc.mchammer))
    else:
        missing_sdms = dc.get_mchammer_workspace()
        if len(missing_sdms) > 0:
            agg.error("Missing SDMs for {0}: ".format(dc.mchammer) +
                      ', '.join(list(missing_sdms)))
        else:
            agg.ok("No missing recent SDMs")


def check_faults():
    r""" The functional called when this file is run as a script.

    :return:
    """
    args = get_settings()

    # print('checking faults on {0}:'.format(args.profile))

    aggregator = BulkReporter()
    # slack = SlackReporter()
    aggregator.add_aggregator(ConsoleReporter())
    # aggregator.add_aggregator(slack)

    try:
        check_services(aggregator, args.profile)
        check_queues(aggregator, args.profile)
        check_last_xxx(aggregator, args.profile)
        check_directories(aggregator, args.profile)

        # slack.publish()

    except Exception as cmd_ex:
        die_message('Exception caught running faultchecker: {0} - {1}\n'
                    .format(type(cmd_ex).__name__, cmd_ex), get_parser(), 3)


if __name__ == '__main__':
    check_faults()
